library digital_3g_common;

import 'package:clipboard/clipboard.dart';
import 'package:flutter/material.dart';
import 'package:package_info/package_info.dart';
import 'package:url_launcher/url_launcher.dart';

Future<void> showPackageAboutDialog(BuildContext context, {Icon? applicationIcon}) async {
  final packageInfo = await PackageInfo.fromPlatform();
  showAboutDialog(
    context: context,
    applicationName: packageInfo.appName,
    applicationIcon: applicationIcon,
    applicationLegalese: '© 2021 AStA der Universität Paderborn',
    applicationVersion: '${packageInfo.version}+${packageInfo.buildNumber}',
  );
}

Future<void> showWebsite(BuildContext context, String url) async {
  final success = await launch(url);

  if (!success) {
    ScaffoldMessenger.of(context).showSnackBar(SnackBar(
      content: const Text('Leider konnte die Website nicht aufgerufen werden'),
      action: SnackBarAction(onPressed: () => FlutterClipboard.copy(url), label: 'Link kopieren'),
    ));
  }
}

class CirclePainter extends CustomPainter {
  final Color color;

  CirclePainter({this.color = Colors.white});

  @override
  void paint(Canvas canvas, Size size) {
    final _paint = Paint()
      ..color = color
      ..strokeWidth = 16
      ..style = PaintingStyle.fill;
    canvas.drawOval(
      Rect.fromLTWH(0, 0, size.width, size.height),
      _paint,
    );
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) => false;
}

class ColorCircle extends StatelessWidget {
  final Color color;
  final double radius;
  final Function()? onTap;
  final BorderSide borderSide;
  final bool inverse;
  final EdgeInsets padding;

  final bool disabled;

  const ColorCircle({
    Key? key,
    required this.color,
    this.radius = 16.0,
    this.onTap,
    this.borderSide = const BorderSide(width: 1.0, color: Colors.grey),
    this.inverse = false,
    this.padding = const EdgeInsets.all(0.0),
    this.disabled = false,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: radius * 2,
      width: radius * 2,
      child: disabled
          ? Icon(
              Icons.cancel_rounded,
              size: 128.0 * MediaQuery.of(context).devicePixelRatio,
              color: Colors.red,
            )
          : MaterialButton(
              onPressed: onTap,
              shape: CircleBorder(side: borderSide),
              color: inverse ? Colors.black87 : color,
              disabledColor: inverse ? Colors.black87 : color,
              splashColor: Colors.white54,
              padding: padding,
              child: Image.asset(
                'assets/towers.png',
                package: 'digital_3g_common',
                filterQuality: FilterQuality.medium,
                color: inverse ? color : Colors.black87,
                colorBlendMode: BlendMode.srcATop,
                fit: BoxFit.contain,
                scale: MediaQuery.of(context).devicePixelRatio,
                alignment: Alignment.topCenter,
              ),
            ),
    );
  }
}

class PaddedCard extends StatelessWidget {
  final Widget? child;

  final EdgeInsetsGeometry? margin;
  final EdgeInsetsGeometry padding;

  const PaddedCard({
    Key? key,
    this.child,
    this.margin = const EdgeInsets.all(8.0),
    this.padding = const EdgeInsets.all(8.0),
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Card(
      margin: margin,
      child: Padding(
        padding: padding,
        child: child,
      ),
    );
  }
}
