import 'dart:convert';

import 'package:http/http.dart';
import 'package:logging/logging.dart';
import 'package:oauth2/oauth2.dart' as oauth2;

import '../models/auth_request.dart';
import '../models/user.dart';

class AuthApi {
  final _log = Logger('AuthApi');
  final String _baseUrl;
  Client _client;

  User? _user;

  AuthApi(this._client, this._baseUrl);

  Client get client => _client;

  User? get user => _user;

  void loadClient(oauth2.Client client) {
    _user ??= User(username: '');
    _client = client;
    _user!.credentials = client.credentials;
  }

  Future<void> logout() async {
    await _client.post(Uri.parse('$_baseUrl/oauth/revoke'));

    return;
  }

  Future<Client> postAuthToken(AuthRequest request) async {
    final authorizationEndpoint = Uri.parse('$_baseUrl/oauth/token');

    _user ??= User(username: request.username);

    final clientResponse = await _client.post(
      Uri.parse('$_baseUrl/get_client'),
      body: json.encode({
        'username': request.username,
        'scope': request.scopeString,
        'grant_type': request.grantType
      }),
      headers: {'Content-Type': 'application/json'},
    );
    if (clientResponse.statusCode != 200) {
      _log.severe(clientResponse.body);
      throw Exception('client could not be created');
    }

    final clientData = json.decode(clientResponse.body);

    final client = await oauth2.resourceOwnerPasswordGrant(
      authorizationEndpoint,
      request.username,
      request.password,
      identifier: clientData['client_id'],
      secret: clientData['client_secret'],
      httpClient: _client,
      scopes: request.scopes.map((Scope s) => s.toString().split('.').last).toList(),
    );

    _user!.credentials = client.credentials;

    _client = client;
    return client;
  }

  Future<User?> register(
      {required User user, required String password, required String code}) async {
    final response = await _client.post(
      Uri.parse('$_baseUrl/register'),
      headers: {'Content-Type': 'application/json'},
      body: json.encode({
        'username': user.username,
        'password': password,
        'code': code,
        if (user.name != null) 'name': user.name,
        if (user.surname != null) 'surname': user.surname,
      }),
    );

    if (response.statusCode == 200) {
      return User.fromMap(json.decode(response.body));
    } else {
      return null;
    }
  }
}
