class RibbonColor {
  final int? id;
  int hex;
  bool inverse;
  String? name;

  RibbonColor({this.id, required this.hex, this.name, this.inverse = false});

  factory RibbonColor.fromMap(Map map) => RibbonColor(
        id: map['id'] as int,
        hex: int.parse(map['hex'] as String, radix: 16),
        inverse: map['inverse'] != null ? map['inverse'] as bool : false,
        name: map['name'],
      );

  Map<String, dynamic> get toMap => {
        'id': id,
        'hex': hex.toRadixString(16),
        'name': name,
        'inverse': inverse,
      };

  String get toRadixString =>
      '0x${(hex - 0xFF000000).toRadixString(16).toUpperCase().padLeft(6, '0')}';

  RibbonColor copyWith({
    int? id,
    int? hex,
    bool? inverse,
    String? name,
  }) {
    return RibbonColor(
      id: id ?? this.id,
      hex: hex ?? this.hex,
      inverse: inverse ?? this.inverse,
    );
  }

  @override
  String toString() => '[RibbonColor $toMap]';
}
