import 'package:oauth2/oauth2.dart';

import 'auth_request.dart' show Scope;

class User {
  final int? id;
  String username;

  String? name;
  String? surname;
  String? uuid;

  Credentials? credentials;
  String? scope;

  User({
    this.id,
    required this.username,
    this.name,
    this.surname,
    this.uuid,
    this.scope = 'user',
  });

  factory User.fromMap(Map map) => User(
        id: map['id'] as int,
        username: map['username'] as String,
        name: map['name'],
        surname: map['surname'],
        uuid: map['uuid'] as String,
        scope: map['scope'],
      );

  String get fullName => name != null && name!.isNotEmpty && surname != null && surname!.isNotEmpty
      ? '$name $surname'
      : username;

  Scope? get scopeObj {
    if (scope == 'user') {
      return Scope.user;
    } else if (scope == 'admin') {
      return Scope.admin;
    }
    return null;
  }

  Map<String, dynamic> get toMap => <String, dynamic>{
        'id': id,
        'username': username,
        'name': name,
        'surname': surname,
        'scope': scope,
      };

  Map<String, dynamic> get toMapFull => <String, dynamic>{
        ...toMap,
        'uuid': uuid,
      };

  void addAuthData(Map map) {
    credentials = Credentials(
      map['access_token'] as String,
      expiration: DateTime.now().add(Duration(hours: map['expiresIn'] as int)),
      refreshToken: map['refresh_token'] as String,
    );
  }

  @override
  String toString() => '[User $toMapFull]';
}
