import 'package:digital_3g_common/backend_switcher.dart';
import 'package:digital_3g_common/theme.dart';
import 'package:digital_3g_common/ui.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:intl/intl.dart';
import 'package:provider/provider.dart';
import 'package:table_calendar/table_calendar.dart';

import '../api/admin_api.dart';
import '../api/auth_api.dart';
import '../models/color_allocation.dart';
import '../models/ribbon_color.dart';
import '../widgets/color_dialog.dart';

final kFirstDay = DateTime(kToday.year, kToday.month - 1, kToday.day);
final kLastDay = DateTime(kToday.year, kToday.month + 6, kToday.day);
final kToday = DateTime.now();

class ColorAllocationsPage extends StatefulWidget {
  const ColorAllocationsPage({Key? key}) : super(key: key);

  @override
  _ColorAllocationsPageState createState() => _ColorAllocationsPageState();
}

class _ColorAllocationsPageState extends State<ColorAllocationsPage> {
  AdminApi? _adminApi;

  List<ColorAllocation> _allocations = [];
  List<RibbonColor> _colors = [];

  CalendarFormat _calendarFormat = CalendarFormat.month;
  DateTime _focusedDay = DateTime.now();
  DateTime? _selectedDay = DateTime.now();

  final DateFormat _dateFormat = DateFormat.yMMMMEEEEd();

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
          title: const Text('Farbzuweisungen'),
        ),
        body: Center(
            child: ConstrainedBox(
          constraints: const BoxConstraints(maxWidth: 640.0),
          child: RefreshIndicator(
            onRefresh: _onUpdateColors,
            child: ListView(
              children: [
                PaddedCard(
                  child: TableCalendar(
                    locale: 'de_DE',
                    firstDay: kFirstDay,
                    lastDay: kLastDay,
                    focusedDay: _focusedDay,
                    calendarFormat: _calendarFormat,
                    availableCalendarFormats: {_calendarFormat: 'Monat'},
                    headerStyle:
                        HeaderStyle(titleTextStyle: Theme.of(context).textTheme.headline6!),
                    daysOfWeekHeight: 20.0,
                    daysOfWeekStyle: DaysOfWeekStyle(
                      weekdayStyle: Theme.of(context).textTheme.subtitle1!,
                      weekendStyle: Theme.of(context).textTheme.subtitle1!,
                    ),
                    startingDayOfWeek: StartingDayOfWeek.monday,
                    calendarBuilders: CalendarBuilders(
                      outsideBuilder: (BuildContext context, DateTime a, DateTime b) {
                        ColorAllocation? alloc;
                        final allocs =
                            _allocations.where((ColorAllocation e) => isSameDay(e.day, a));
                        if (allocs.isNotEmpty) {
                          alloc = allocs.first;
                        }
                        return Center(
                          child: Container(
                            width: 38.0,
                            height: 38.0,
                            decoration: BoxDecoration(
                              boxShadow: [
                                if (alloc != null)
                                  BoxShadow(
                                      color: Colors.black54.withAlpha(64),
                                      blurRadius: 4.0,
                                      spreadRadius: 0.0),
                              ],
                              shape: BoxShape.circle,
                              color: alloc != null ? Color(alloc.colorHex).withAlpha(64) : null,
                              border: alloc?.colorInverse ?? false
                                  ? Border.all(
                                      color: Colors.black.withAlpha(64),
                                      width: 4.2,
                                    )
                                  : null,
                            ),
                            padding: const EdgeInsets.only(bottom: 4.0),
                            alignment: Alignment.center,
                            child: Text(a.day.toString(),
                                style: Theme.of(context).textTheme.subtitle1!.copyWith(
                                    color: alloc != null
                                        ? (Color(alloc.colorHex).computeLuminance() < 0.5)
                                            ? Colors.white
                                            : Colors.black54
                                        : Colors.black54)),
                          ),
                        );
                      },
                      selectedBuilder: (BuildContext context, DateTime a, DateTime b) {
                        final allocs =
                            _allocations.where((ColorAllocation e) => isSameDay(e.day, a));
                        ColorAllocation? alloc;
                        if (allocs.isNotEmpty) {
                          alloc = allocs.first;
                        }
                        return Center(
                          child: Container(
                            width: 38.0,
                            height: 38.0,
                            decoration: BoxDecoration(
                              boxShadow: const [
                                BoxShadow(color: magenta, blurRadius: 6.0, spreadRadius: 2.0),
                              ],
                              shape: BoxShape.circle,
                              border: alloc?.colorInverse ?? false
                                  ? Border.all(
                                      color: Colors.black87,
                                      width: 4.2,
                                    )
                                  : null,
                              color: alloc != null ? Color(alloc.colorHex) : null,
                            ),
                            padding: const EdgeInsets.only(bottom: 4.0),
                            alignment: Alignment.center,
                            child: Text(a.day.toString(),
                                style: Theme.of(context).textTheme.subtitle1!.copyWith(
                                    color: alloc != null
                                        ? (Color(alloc.colorHex).computeLuminance() < 0.5)
                                            ? Colors.white
                                            : Colors.black
                                        : null)),
                          ),
                        );
                      },
                      todayBuilder: (BuildContext context, DateTime a, DateTime b) {
                        final allocs =
                            _allocations.where((ColorAllocation e) => isSameDay(e.day, a));
                        ColorAllocation? alloc;
                        if (allocs.isNotEmpty) {
                          alloc = allocs.first;
                        }
                        return Center(
                          child: Container(
                            width: 38.0,
                            height: 36.0,
                            decoration: BoxDecoration(
                              boxShadow: const [
                                BoxShadow(color: secondary, blurRadius: 4.0, spreadRadius: 2.0),
                              ],
                              shape: BoxShape.circle,
                              border: alloc?.colorInverse ?? false
                                  ? Border.all(
                                      color: Colors.black87,
                                      width: 4.2,
                                    )
                                  : null,
                              color: alloc != null ? Color(alloc.colorHex) : null,
                            ),
                            padding: const EdgeInsets.only(bottom: 4.0),
                            alignment: Alignment.center,
                            child: Text(a.day.toString(),
                                style: Theme.of(context).textTheme.subtitle1!.copyWith(
                                    color: alloc != null
                                        ? (Color(alloc.colorHex).computeLuminance() < 0.5)
                                            ? Colors.white
                                            : Colors.black87
                                        : null)),
                          ),
                        );
                      },
                      defaultBuilder: (BuildContext context, DateTime a, DateTime b) {
                        final allocs =
                            _allocations.where((ColorAllocation e) => isSameDay(e.day, a));
                        ColorAllocation? alloc;
                        if (allocs.isNotEmpty) {
                          alloc = allocs.first;
                        }
                        return Center(
                          child: Container(
                            width: 38.0,
                            height: 38.0,
                            decoration: BoxDecoration(
                              boxShadow: [
                                if (alloc != null)
                                  const BoxShadow(
                                      color: Colors.black54, blurRadius: 4.0, spreadRadius: 0.0),
                              ],
                              color: alloc != null ? Color(alloc.colorHex) : null,
                              shape: BoxShape.circle,
                              border: alloc?.colorInverse ?? false
                                  ? Border.all(
                                      color: Colors.black87,
                                      width: 4.2,
                                    )
                                  : null,
                            ),
                            padding: const EdgeInsets.only(bottom: 4.0),
                            alignment: Alignment.center,
                            child: Text(
                              a.day.toString(),
                              style: Theme.of(context).textTheme.subtitle1!.copyWith(
                                  color: alloc != null
                                      ? (Color(alloc.colorHex).computeLuminance() < 0.5)
                                          ? Colors.white
                                          : Colors.black
                                      : null),
                            ),
                          ),
                        );
                      },
                    ),
                    selectedDayPredicate: (day) {
                      return isSameDay(_selectedDay, day);
                    },
                    onDaySelected: (selectedDay, focusedDay) {
                      if (!isSameDay(_selectedDay, selectedDay)) {
                        setState(() {
                          _selectedDay = selectedDay;
                          _focusedDay = focusedDay;
                        });
                      }
                    },
                    onFormatChanged: (format) {
                      if (_calendarFormat != format) {
                        setState(() => _calendarFormat = format);
                      }
                    },
                    onPageChanged: (focusedDay) {
                      _focusedDay = focusedDay;
                    },
                  ),
                ),
                Builder(
                  builder: (BuildContext context) {
                    final allocs =
                        _allocations.where((ColorAllocation e) => isSameDay(e.day, _selectedDay));
                    ColorAllocation? alloc;
                    if (allocs.isNotEmpty) {
                      alloc = allocs.first;
                    }
                    return PaddedCard(
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          ListTile(
                            title: const Text('Farbe für'),
                            subtitle: Text(
                              _dateFormat.format(_selectedDay!),
                              style: Theme.of(context).textTheme.headline6,
                            ),
                            trailing: alloc?.colorHex != null
                                ? ColorCircle(
                                    color: Color(alloc!.colorHex),
                                    radius: 24.0,
                                    inverse: alloc.colorInverse,
                                    padding: const EdgeInsets.all(8.0),
                                  )
                                : const SizedBox(height: 48.0, width: 48.0),
                          ),
                          const Divider(),
                          ListTile(
                            title: const Text('Farbe wählen:'),
                            trailing: IconButton(
                              onPressed: _onAddColor,
                              icon: const Icon(Icons.add),
                              tooltip: 'Neue Farbe hinzufügen',
                            ),
                          ),
                          GridView.count(
                            crossAxisCount: MediaQuery.of(context).size.width >= 768.0 ? 6 : 4,
                            shrinkWrap: true,
                            padding: const EdgeInsets.symmetric(horizontal: 16.0, vertical: 8.0),
                            childAspectRatio: 1.0,
                            mainAxisSpacing: 16.0,
                            crossAxisSpacing: 16.0,
                            physics: const NeverScrollableScrollPhysics(),
                            children: [
                              IconButton(
                                icon: const Icon(Icons.cancel),
                                iconSize: 42.0,
                                onPressed: () => _onDeleteColor(_selectedDay!),
                                tooltip: 'Zuweisung löschen',
                              ),
                              ..._colors.map(
                                (e) => Tooltip(
                                  padding: const EdgeInsets.all(2.0),
                                  height: 20.0,
                                  verticalOffset: 36.0,
                                  message:
                                      '${e.name ?? e.toRadixString}\n${e.inverse ? 'Invertiertes Logo' : 'Normales Logo'}',
                                  child: ColorCircle(
                                    color: Color(e.hex),
                                    onTap: () => _onSelectColor(_selectedDay!, e.id!),
                                    inverse: e.inverse,
                                    padding: const EdgeInsets.all(12.0),
                                  ),
                                ),
                              ),
                            ],
                          )
                        ],
                      ),
                    );
                  },
                ),
              ],
            ),
          ),
        )));
  }

  @override
  void initState() {
    super.initState();
    _initApi();
  }

  void _initApi() async {
    final _client = Provider.of<AuthApi>(context, listen: false).client;
    _adminApi = AdminApi(_client, BlocProvider.of<BackendSwitcher>(context).state);

    _onUpdateColors();
    super.didChangeDependencies();
  }

  void _onAddColor() async {
    final result = await showDialog<ColorDialogResult>(
      context: context,
      builder: (BuildContext context) => const ColorDialog(),
    );

    if (!(result?.save ?? false)) return;

    try {
      _colors.add((await _adminApi!.addColor(result!.color))!);
    } catch (e) {
      ScaffoldMessenger.of(context).showSnackBar(
        const SnackBar(content: Text('Farbe konnte nicht erstellt werden')),
      );
    }
    setState(() {
      _colors;
    });
  }

  Future<void> _onDeleteColor(DateTime day) {
    try {
      final alloc = _allocations.singleWhere((element) => isSameDay(element.day, day));
      _adminApi!.deleteColorAllocation(alloc.id!);
      setState(() => _allocations.removeWhere((element) => element.id == alloc.id));
      return Future.value(null);
    } catch (e) {
      return Future.value(null);
    }
  }

  void _onSelectColor(DateTime day, int id) async {
    if (_allocations.where((element) => isSameDay(element.day, day)).isEmpty) {
      final color = _colors.singleWhere((element) => element.id == id);
      _allocations.add(
        (await _adminApi!.addColorAllocation(
          ColorAllocation(
            day: day,
            colorId: color.id!,
            colorHex: color.hex,
          ),
        ))!,
      );
    } else {
      final color = _colors.singleWhere((element) => element.id == id);
      final alloc = _allocations
          .singleWhere((element) => isSameDay(element.day, day))
          .copyWith(colorId: color.id, colorHex: color.hex);
      final newAlloc = await _adminApi!.updateColorAllocation(alloc);
      _allocations.removeWhere((element) => element.id == newAlloc!.id);
      setState(() => _allocations.add(newAlloc!));
    }
    setState(() => _allocations);
  }

  Future<void> _onUpdateColors() async {
    _colors = (await _adminApi!.getColors()) ?? [];
    _allocations = (await _adminApi!.getColorAllocations()) ?? [];
    setState(() => _colors);
  }
}
