import 'package:digital_3g_common/content_dialog.dart';
import 'package:digital_3g_common/theme.dart';
import 'package:flutter/material.dart';

import '../models/user.dart';
import 'scope_selector.dart';

class UserDialog extends StatefulWidget {
  final User? user;

  const UserDialog({Key? key, this.user}) : super(key: key);

  @override
  _UserDialogState createState() => _UserDialogState();
}

class UserResult {
  bool save;
  User? user;
  String? password;

  UserResult({required this.save, this.user, this.password});
}

class _UserDialogState extends State<UserDialog> {
  String? _password;
  User? _user;

  bool _showPassword = false;

  final GlobalKey<FormState> _formKey = GlobalKey();

  @override
  Widget build(BuildContext context) {
    return ContentDialog(
      title: Text(widget.user != null ? 'Nutzer bearbeiten' : 'Nutzer hinzufügen'),
      children: [
        Padding(
          padding: const EdgeInsets.only(bottom: 8.0),
          child: Form(
            key: _formKey,
            child: Column(
              children: [
                TextFormField(
                  decoration: const InputDecoration(labelText: 'Nutzername'),
                  onChanged: (String input) => _user!.username = input,
                  controller: TextEditingController(text: _user!.username),
                  validator: (String? input) =>
                      input == null || input.isEmpty ? 'Pflichtfeld' : null,
                ),
                if (widget.user == null)
                  TextFormField(
                    decoration: InputDecoration(
                      labelText: 'Passwort',
                      suffix: IconButton(
                        onPressed: () => setState(() => _showPassword = !_showPassword),
                        icon: Icon(
                          Icons.remove_red_eye_rounded,
                          color: _showPassword ? secondary : darkGrey,
                        ),
                        constraints: const BoxConstraints(maxWidth: 24.0),
                        padding: EdgeInsets.zero,
                      ),
                    ),
                    onChanged: (String input) => _password = input,
                    controller: TextEditingController(text: _password ?? ''),
                    validator: (String? input) => (input?.length ?? 0) < 5
                        ? 'Das Passwort muss mindestens 5 Zeichen enthalten'
                        : input == null || input.isEmpty
                            ? 'Pflichtfeld'
                            : null,
                    obscureText: !_showPassword,
                  ),
                TextFormField(
                  decoration: const InputDecoration(labelText: 'Vorname'),
                  onChanged: (String input) => _user!.name = input,
                  controller: TextEditingController(text: _user!.name),
                ),
                TextFormField(
                  decoration: const InputDecoration(labelText: 'Nachname'),
                  onChanged: (String input) => _user!.surname = input,
                  controller: TextEditingController(text: _user!.surname),
                ),
                ListTile(
                  title: const Text('Rolle'),
                  trailing: SizedBox(
                    width: 128.0,
                    child: ScopeSelector(
                      onChanged: (Scope? input) =>
                          input != null ? _user!.scope = input.toString().split('.').last : null,
                      value: _user!.scopeObj ?? Scope.user,
                      disabled: widget.user != null,
                    ),
                  ),
                ),
              ],
            ),
          ),
        ),
        const Divider(),
      ],
      actions: [
        ElevatedButton(
          onPressed: () => Navigator.of(context).pop(UserResult(save: false)),
          child: const Text('Abbrechen'),
        ),
        ElevatedButton(
          onPressed: () {
            if (_formKey.currentState?.validate() ?? false) {
              Navigator.of(context).pop(UserResult(save: true, user: _user, password: _password));
            }
          },
          child: const Text('Speichern'),
        ),
      ],
    );
  }

  @override
  void initState() {
    super.initState();
    if (widget.user != null) {
      _user = widget.user;
    } else {
      _user = User(username: '');
    }
  }
}
