import 'dart:io';

import 'package:digital_3g_common/backend_switcher.dart';
import 'package:digital_3g_common/ui.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_phoenix/flutter_phoenix.dart';
import 'package:path_provider/path_provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'verification_bloc.dart';

class SettingsPage extends StatefulWidget {
  const SettingsPage({Key? key}) : super(key: key);

  @override
  State<SettingsPage> createState() => _SettingsPageState();
}

class _SettingsPageState extends State<SettingsPage> {
  BackendSwitcher? _backendSwitcher;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Einstellungen'),
      ),
      backgroundColor: Colors.white,
      body: BlocBuilder<VerificationBloc, VerificationState>(
        builder: (BuildContext context, VerificationState state) => ListView(
          padding: const EdgeInsets.all(8.0),
          children: [
            if (state != VerificationState.noTicket)
              ListTile(
                title: const Text('Nachweis löschen'),
                trailing: const Icon(Icons.delete_forever_rounded),
                onTap: _onDeleteTicket,
              ),
            if (state != VerificationState.noTicket) const Divider(),
            BlocBuilder<BackendSwitcher, String>(
              builder: (BuildContext context, String state) => ListTile(
                title: const Text('Debug-Backend nutzen'),
                trailing: Switch(
                  value: state == _backendSwitcher!.development,
                  onChanged: _onSwitchBackend,
                ),
              ),
            ),
            const Divider(),
            ListTile(
              title: const Text('Datenschutzinformationen'),
              trailing: const Icon(Icons.security_rounded),
              onTap: () => showWebsite(context, 'https://go.upb.de/datenschutz_3digi'),
            ),
            ListTile(
              title: const Text('Hilfe zur App'),
              trailing: const Icon(Icons.help_outline_rounded),
              onTap: () => showWebsite(context, 'https://go.upb.de/hilfe_3digi'),
            ),
            const Divider(),
            ListTile(
              title: const Text('Über diese App'),
              trailing: const Icon(Icons.info_outline_rounded),
              onTap: () => showPackageAboutDialog(context),
            ),
          ],
        ),
      ),
    );
  }

  @override
  void didChangeDependencies() {
    _backendSwitcher = BlocProvider.of<BackendSwitcher>(context);
    super.didChangeDependencies();
  }

  void _onDeleteTicket() async {
    final result = await showDialog<bool>(
      context: context,
      builder: (BuildContext context) => AlertDialog(
        title: const Text('Der Nachweis ist danach endgültig gelöscht. Bist du dir sicher?'),
        actions: [
          MaterialButton(
            onPressed: () => Navigator.of(context).pop(true),
            child: const Text('Ja'),
          ),
          ElevatedButton(
            onPressed: () => Navigator.of(context).pop(false),
            child: const Text('Nein'),
          ),
        ],
      ),
    );

    if (result ?? false) BlocProvider.of<VerificationBloc>(context).onRevoke();

    Navigator.of(context).pop();
  }

  void _onSwitchBackend(bool input) async {
    _backendSwitcher!.add(input ? SwitchDevelopment() : SwitchProduction());
    final prefs = await SharedPreferences.getInstance();
    final lastVerification = File((await getApplicationDocumentsDirectory()).path + '/last');
    if (lastVerification.existsSync()) {
      await lastVerification.delete();
    }
    prefs.setString('digital_3g_backend_mode', input ? 'development' : 'production');
    await Future.delayed(const Duration(milliseconds: 400));
    Phoenix.rebirth(context);
  }
}
